@@ -1,20 +1,21 @@
 import logging
+import re
 import threading
-from typing import List
 from dataclasses import dataclass
+from typing import List, Optional
 
+from JABWrapper.jab_types import AccessibleContextInfo, JavaObject
 from JABWrapper.jab_wrapper import JavaAccessBridgeWrapper
-from JABWrapper.jab_types import JavaObject, AccessibleContextInfo
-from JABWrapper.utils import log_exec_time, retry_callback, SearchElement
-from JABWrapper.parsers.parser_if import Parser
-from JABWrapper.parsers.text_parser import AccessibleTextParser
-from JABWrapper.parsers.value_parser import AccessibleValueParser
 from JABWrapper.parsers.actions_parser import AccessibleActionsParser
+from JABWrapper.parsers.hypertext_parser import AccessibleHypertextParser
 from JABWrapper.parsers.icon_parser import AccessibleIconParser
 from JABWrapper.parsers.keybind_parser import AccessibleKeyBindingsParser
-from JABWrapper.parsers.hypertext_parser import AccessibleHypertextParser
-from JABWrapper.parsers.table_parser import AccessibleTableParser
+from JABWrapper.parsers.parser_if import Parser
 from JABWrapper.parsers.selection_parser import AccessibleSelectionParser
+from JABWrapper.parsers.table_parser import AccessibleTableParser
+from JABWrapper.parsers.text_parser import AccessibleTextParser
+from JABWrapper.parsers.value_parser import AccessibleValueParser
+from JABWrapper.utils import SearchElement, log_exec_time, retry_callback
 
 
 @dataclass
@@ -33,20 +34,39 @@ class NodeLocator:
 
 
 class ContextNode:
-    def __init__(self, jab_wrapper: JavaAccessBridgeWrapper, context: JavaObject, lock, ancestry: int = 0, parse_children=True) -> None:
+    def __init__(
+        self,
+        jab_wrapper: JavaAccessBridgeWrapper,
+        context: JavaObject,
+        lock: threading.RLock,
+        ancestry: int = 0,
+        parse_children: bool = True,
+        max_depth: Optional[int] = None,
+        parent: Optional["ContextNode"] = None,
+    ) -> None:
         self._jab_wrapper = jab_wrapper
         self._lock = lock
         self.ancestry = ancestry
         self.context: JavaObject = context
+        self._should_parse_children = parse_children
+        self._max_depth = max_depth
+
         self.state = None
         self.visible_children_count = 0
         self.virtual_accessible_name = None
-        self._parse_context()
-        self.children: list[ContextNode] = []
-        if parse_children:
-            self._parse_children()
+        self._children: list[ContextNode] = []
 
-    def _parse_context(self) -> None:
+        self.parent: Optional[ContextNode] = parent
+
+        # Populate the element with data and its children if enabled.
+        self.refresh()
+
+    @property
+    def children(self):
+        with self._lock:
+            return self._children
+
+    def parse_context(self) -> None:
         logging.debug(f"Parsing element={self.context}")
         self._aci: AccessibleContextInfo = self._jab_wrapper.get_context_info(self.context)
         logging.debug(f"Parsed element info={self._aci}")
@@ -60,7 +80,16 @@ class ContextNode:
         self.table = AccessibleTableParser(self._aci)
         self.icons = AccessibleIconParser(self._aci)
         self.selections = AccessibleSelectionParser(self._aci)
-        self._parsers: List[Parser] = [self.text, self.value, self.actions, self.keybinds, self.hypertext, self.table, self.icons, self.selections]
+        self._parsers: List[Parser] = [
+            self.text,
+            self.value,
+            self.actions,
+            self.keybinds,
+            self.hypertext,
+            self.table,
+            self.icons,
+            self.selections,
+        ]
         [parser.parse(self._jab_wrapper, self.context) for parser in self._parsers]
 
     @property
@@ -96,30 +125,56 @@ class ContextNode:
         self._aci = context_info
 
     def _parse_children(self) -> None:
+        if self._max_depth is not None and self.ancestry >= self._max_depth:
+            return
+
         for i in range(0, self._aci.childrenCount):
             child_context = self._jab_wrapper.get_child_context(self.context, i)
-            child_node = ContextNode(self._jab_wrapper, child_context, self._lock, self.ancestry + 1)
-            self.children.append(child_node)
+            child_node = ContextNode(
+                self._jab_wrapper,
+                child_context,
+                self._lock,
+                self.ancestry + 1,
+                parse_children=self._should_parse_children,
+                max_depth=self._max_depth,
+                parent=self,
+            )
+            self._children.append(child_node)
+
+    def refresh(self):
+        """Refresh the current element and its children only.
+
+        Useful when you want to refresh just a subtree of elements starting from the
+        current one as root, instead of the entire app.
+        """
+        with self._lock:
+            self.state = None
+            self._children.clear()
+            self.parse_context()
+            if self._should_parse_children:
+                self._parse_children()
 
     def __repr__(self) -> str:
         """
         Returns:
             A string that represents the object tree with detailed Node values.
         """
-        string = (f"{'| ' * self.ancestry}"
-                  f"role:{self.context_info.role}; "
-                  f"name:{self.context_info.name}; "
-                  f"virtual_accessible_name:{self.virtual_accessible_name}; "
-                  f"description:{self.context_info.description}; "
-                  f"ancestry:{self.ancestry}; "
-                  f"state:{self.state}; "
-                  f"states:{self.context_info.states}; "
-                  f"at x:{self.context_info.x} y:{self.context_info.y}; "
-                  f"width:{self.context_info.width}; "
-                  f"height:{self.context_info.height}; "
-                  f"indexInParent:{self.context_info.indexInParent}; "
-                  f"childrenCount:{self.context_info.childrenCount}; "
-                  f"visible_children_count:{self.visible_children_count}")
+        string = (
+            f"{'| ' * self.ancestry}"
+            f"role:{self.context_info.role}; "
+            f"name:{self.context_info.name}; "
+            f"virtual_accessible_name:{self.virtual_accessible_name}; "
+            f"description:{self.context_info.description}; "
+            f"ancestry:{self.ancestry}; "
+            f"state:{self.state}; "
+            f"states:{self.context_info.states}; "
+            f"at x:{self.context_info.x} y:{self.context_info.y}; "
+            f"width:{self.context_info.width}; "
+            f"height:{self.context_info.height}; "
+            f"indexInParent:{self.context_info.indexInParent}; "
+            f"childrenCount:{self.context_info.childrenCount}; "
+            f"visible_children_count:{self.visible_children_count}"
+        )
         for parser in self._parsers:
             string += f"{parser}"
         for child in self.children:
@@ -131,43 +186,47 @@ class ContextNode:
         Returns:
             A string of Node values.
         """
-        string = (f"role:{self.context_info.role}; "
-                  f"name:{self.context_info.name}; "
-                  f"virtual_accessible_name:{self.virtual_accessible_name}; "
-                  f"description:{self.context_info.description}; "
-                  f"ancestry:{self.ancestry}; "
-                  f"state:{self.state}; "
-                  f"states:{self.context_info.states}; "
-                  f"at x:{self.context_info.x} y:{self.context_info.y}; "
-                  f"width:{self.context_info.width}; "
-                  f"height:{self.context_info.height}; "
-                  f"indexInParent:{self.context_info.indexInParent}; "
-                  f"childrenCount:{self.context_info.childrenCount}; "
-                  f"visible_children_count:{self.visible_children_count}")
+        string = (
+            f"role:{self.context_info.role}; "
+            f"name:{self.context_info.name}; "
+            f"virtual_accessible_name:{self.virtual_accessible_name}; "
+            f"description:{self.context_info.description}; "
+            f"ancestry:{self.ancestry}; "
+            f"state:{self.state}; "
+            f"states:{self.context_info.states}; "
+            f"at x:{self.context_info.x} y:{self.context_info.y}; "
+            f"width:{self.context_info.width}; "
+            f"height:{self.context_info.height}; "
+            f"indexInParent:{self.context_info.indexInParent}; "
+            f"childrenCount:{self.context_info.childrenCount}; "
+            f"visible_children_count:{self.visible_children_count}"
+        )
         for parser in self._parsers:
             string += "{}".format(parser)
         return string
 
     def get_search_element_tree(self) -> List[NodeLocator]:
         """
-        Returns node info for all searcheable elements.
+        Returns node info for all searchable elements.
         """
-        nodes = list()
-        nodes.append({
-            "name": self.context_info.name,
-            "description": self.context_info.description,
-            "role": self.context_info.role,
-            "states": self.context_info.states,
-            "indexInParent": self.context_info.indexInParent,
-            "childrenCount": self.context_info.childrenCount,
-            "x": self.context_info.x,
-            "y": self.context_info.y,
-            "width": self.context_info.width,
-            "height": self.context_info.height,
-            "ancestry": self.ancestry
-        })
+        nodes = []
+        nodes.append(
+            NodeLocator(
+                self.context_info.name,
+                self.context_info.description,
+                self.context_info.role,
+                self.context_info.states,
+                self.context_info.indexInParent,
+                self.context_info.childrenCount,
+                self.context_info.x,
+                self.context_info.y,
+                self.context_info.width,
+                self.context_info.height,
+                self.ancestry,
+            )
+        )
         for child in self.children:
-            nodes += child.get_search_element_tree()
+            nodes.extend(child.get_search_element_tree())
         return nodes
 
     def traverse(self):
@@ -182,21 +241,16 @@ class ContextNode:
         if self._jab_wrapper.is_same_object(self.context, context):
             return self
         else:
-            for child in self.children:
+            for child in self._children:
                 node = child._get_node_by_context(context)
                 if node:
                     return node
 
-    def _update_node(self) -> None:
-        self.children = []
-        self._parse_context()
-        self._parse_children()
-
     def _match_attrs(self, search_elements: List[SearchElement]) -> bool:
         for search_element in search_elements:
             attr = getattr(self._aci, search_element.name)
             if isinstance(attr, str) and not search_element.strict:
-                if not attr.startswith(search_element.value):
+                if not re.match(search_element.value, attr):
                     return False
             else:
                 if not attr == search_element.value:
@@ -205,7 +259,7 @@ class ContextNode:
 
     def get_by_attrs(self, search_elements: List[SearchElement]) -> List:
         """
-        Get element with given seach attributes.
+        Get element with given search attributes.
 
         The SearchElement object takes a name of the field and the field value. For example:
 
@@ -219,7 +273,7 @@ class ContextNode:
             found = self._match_attrs(search_elements)
             if found:
                 elements.append(self)
-            for child in self.children:
+            for child in self._children:
                 child_elements = child.get_by_attrs(search_elements)
                 elements.extend(child_elements)
             return elements
@@ -276,20 +330,27 @@ class ContextNode:
         visible_children = []
         logging.debug(f"Expected visible children count={self.visible_children_count}")
         if self.visible_children_count > 0:
-            found_visible_children = self._jab_wrapper.get_visible_children(self.context, 0)
+            visible_children_info = self._jab_wrapper.get_visible_children(self.context, 0)
             logging.debug(f"Found visible children count={self.visible_children_count}")
-            for i in range(0, found_visible_children.returnedChildrenCount):
-                visible_child = ContextNode(self._jab_wrapper, found_visible_children.children[i], self._lock, self.ancestry + 1, False)
+            for i in range(0, visible_children_info.returnedChildrenCount):
+                visible_child = ContextNode(
+                    self._jab_wrapper,
+                    visible_children_info.children[i],
+                    self._lock,
+                    self.ancestry + 1,
+                    parse_children=False,
+                    parent=self,
+                )
                 visible_children.append(visible_child)
         return visible_children
 
 
 class ContextTree:
     @log_exec_time("Init context tree")
-    def __init__(self, jab_wrapper: JavaAccessBridgeWrapper) -> None:
+    def __init__(self, jab_wrapper: JavaAccessBridgeWrapper, max_depth: Optional[int] = None) -> None:
         self._lock = threading.RLock()
         self._jab_wrapper = jab_wrapper
-        self.root = ContextNode(jab_wrapper, jab_wrapper.context, self._lock)
+        self.root = ContextNode(jab_wrapper, jab_wrapper.context, self._lock, parse_children=True, max_depth=max_depth)
         self._register_callbacks()
 
     def __iter__(self):
@@ -349,7 +410,7 @@ class ContextTree:
         with self._lock:
             node: ContextNode = self.root._get_node_by_context(source)
             if node:
-                node._parse_context()
+                node.parse_context()
                 logging.debug(f"Selected text changed for node={node}")
 
     @retry_callback
@@ -357,7 +418,7 @@ class ContextTree:
         with self._lock:
             node: ContextNode = self.root._get_node_by_context(source)
             if node:
-                node._parse_context()
+                node.parse_context()
                 logging.debug(f"Text changed for node={node}")
 
     @retry_callback
@@ -370,7 +431,7 @@ class ContextTree:
         with self._lock:
             node = self.root._get_node_by_context(source)
             if node:
-                node._update_node()
+                node.refresh()
                 logging.debug(f"Visible data changed for node tree={repr(node)}")
 
     @retry_callback
@@ -379,7 +440,9 @@ class ContextTree:
         logging.debug("Property child change event ignored")
 
     @retry_callback
-    def _property_active_descendent_change_cp(self, source: JavaObject, old_child: JavaObject, new_child: JavaObject) -> None:
+    def _property_active_descendent_change_cp(
+        self, source: JavaObject, old_child: JavaObject, new_child: JavaObject
+    ) -> None:
         # The activity status is not stored inside the tree model
         logging.debug("Property active descendent change event ignored")
 
@@ -464,7 +527,7 @@ class ContextTree:
         are generated from the Access Bridge
         """
         if self._jab_wrapper.ignore_callbacks:
-            logging.debug("Ignoring callback regitering for Context Node")
+            logging.debug("Ignoring callback registering for Context Node")
             return
 
         self._jab_wrapper.clear_callbacks()
@@ -480,7 +543,9 @@ class ContextTree:
         self._jab_wrapper.register_callback("property_caret_change", self._property_caret_change_cp)
         self._jab_wrapper.register_callback("property_visible_data_change", self._visible_data_change_cp)
         self._jab_wrapper.register_callback("property_child_change", self._property_child_change_cp)
-        self._jab_wrapper.register_callback("property_active_descendent_change", self._property_active_descendent_change_cp)
+        self._jab_wrapper.register_callback(
+            "property_active_descendent_change", self._property_active_descendent_change_cp
+        )
         self._jab_wrapper.register_callback("property_table_model_change", self._property_table_model_change_cp)
         # Menu event handlers
         self._jab_wrapper.register_callback("menu_selected", self._menu_selected_cp)
@@ -499,7 +564,9 @@ class ContextTree:
         self._jab_wrapper.register_callback("mouse_released", self._mouse_released_cp)
         # Popup menu events
         self._jab_wrapper.register_callback("popup_menu_canceled", self._popup_menu_canceled_cp)
-        self._jab_wrapper.register_callback("popup_menu_will_become_invisible", self._popup_menu_will_become_invisible_cp)
+        self._jab_wrapper.register_callback(
+            "popup_menu_will_become_invisible", self._popup_menu_will_become_invisible_cp
+        )
         self._jab_wrapper.register_callback("popup_menu_will_become_visible", self._popup_menu_will_become_visible_cp)
 
     def get_by_attrs(self, search_elements: List[SearchElement]) -> List[ContextNode]:
